/*
 Standalone Temperature Logger for ATtiny85
 Version: 2.2
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 9/01/2011
 Last Modified: 23/12/2011
 
 Used to monitor temperature every x minutes/hours which is saved 
 to the built in EEPROM. You can transfer the data by connecting via 
 USB to your computer.
 
 Remember to change your fuse bits so the ATtiny85 uses 16MHz PLL and divide clock by 8
 avrdude -p ATtiny85 -c usbtiny -U lfuse:w:0x61:m -U hfuse:w:0xdf:m -U efuse:w:0xff:m 
 
 2.7V Brown-out Detection is optional and is only recommended for CR2032 batteries that 
 are new or near new condition (-U hfuse:w:0xdd:m)
 
 */

//                  +-\/-+
// Ain0 (D 5) PB5  1|o   |8  Vcc
// Ain3 (D 3) PB3  2|    |7  PB2 (D 2) Ain1
// Ain2 (D 4) PB4  3|    |6  PB1 (D 1) pwm1
//            GND  4|    |5  PB0 (D 0) pwm0
//                  +----+

int thermistorPin = 3; // Analog input 3
#define ledPin PB0 // Output
#define buttonPin PB4 // Input
#define usbminusPin PB1 // USB D-

#define F_CPU 16500000

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <math.h>
#include <util/delay.h>
#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"
#include "setup.c"

volatile boolean f_pin = 0; // Pin change variable
volatile boolean buttonPressed = 0; // Keep track of button presses and releases

int functionSelect = WAITINPUT;
int dataCount = 1;
int delayTime = -1;

// Send the next digit from the EEPROM address
int sendnextDigit(int eepromAddress) {
	uchar digit;
	
	int value = eeprom_read_byte((uint8_t*) eepromAddress);
	
	if (value == 0) {
		return 1;
	}
	
	value -= 130; // Minus 130 from value to get real temperature
	
	// Check if value is less than 0, if so convert it to positive number and include the negative sign at the end
	boolean isNegative = false;
	if (value < 0) {
		value = value - (value * 2);
		isNegative = true;
	}
	
	nextDigit = &valueBuffer[sizeof(valueBuffer)];
	*--nextDigit = 0xff;
	*--nextDigit = 0;
	*--nextDigit = KEY_RETURN;
	
	// Break up the number and convert to USB HID values
	do{	
		digit = value % 10;
		value /= 10;
			*--nextDigit = 0;
		if (digit == 0){
			*--nextDigit = KEY_0;
		}
		else{
			*--nextDigit = KEY_1 - 1 + digit;
		}
	} while (value != 0);
	
	if (isNegative == true) {
		*--nextDigit = 0;
		*--nextDigit = KEY_MINUS;
	}
	
	return 0;
}

// Transfers the EEPROM data to USB. Stops if EEPROM value is 0 or the end of the EEPROM is reached (512).
void transferData(void) {
	boolean inusbTransfer = true;
	int readCount = 1;
	while (inusbTransfer == true) {
		usbPoll();
		if(usbInterruptIsReady() && nextDigit != NULL){ // We can send another digit
			buildReport();
			usbSetInterrupt(reportBuffer, sizeof(reportBuffer));
			if(*++nextDigit == 0xff) { // This was terminator character
				nextDigit = NULL;
			}
		}
		if (nextDigit == NULL) {
			if (readCount < 512) {
				int checkforZero = sendnextDigit(readCount);
				if (checkforZero == true) { // End if 0
					inusbTransfer = false;
				}
				readCount++;
			}
			else {
				inusbTransfer = false;
			}
		}
	}
}

// Blink the LED x number of times with the pause calculated by the watchdog timer selected and skip the LED turn off delay if needed
void blinkLed (byte times, byte wdttime, byte skipLedoffdelay) {
	for (int x = 0; x < times; x++) {
		PORTB |= (1<<ledPin); // LED on
		setup_watchdog(wdttime);
		system_sleep();
		
		PORTB &= ~(1<<ledPin); // LED off	
		if (skipLedoffdelay == false) {
			setup_watchdog(wdttime);
			system_sleep();
		}
	}
	turnOffwatchdog();
}

int main(void) {
	setup();
	
	DDRB |= (1<<ledPin); // Set Outputs
	DDRB &= ~((1<<buttonPin) | (1<<usbminusPin)); // Set Inputs
	
	// Pin interrupt setup
	sbi(GIMSK,PCIE); // Enable pin change interrupt
	sbi(PCMSK,PCINT4); // Apply interrupt to button pin
	
	while(1) {
		// Wait for input
		if (functionSelect == WAITINPUT) {
			
			// Read delay time
			if (delayTime == -1) {
				delayTime = eeprom_read_byte((uint8_t*) 0);
				if (delayTime > 10) { // Not a normal value
					delayTime = 0;
				}
			}
			
			// Turn off LED and reset buttonPressed before going to sleep
			PORTB &= ~(1<<ledPin);
			buttonPressed = 0;
			system_sleep();
			
			// Button was pressed, first test we are connected via USB by checking if the D- pin is high
			int usbConnected = PINB & (1<<usbminusPin);
			if (usbConnected) { // Send data from EEPROM to USB
				PORTB |= (1<<ledPin);
				
				// Increase clock speed to 16MHz
				CLKPR = (1<<CLKPCE); // Prescaler enable
				CLKPR = 0; // Clock division factor 1 (0000)
				
				startUSB();
				transferData();
				
				// Decrease clock speed to 1MHz
				CLKPR = (1<<CLKPCE); // Prescaler enable
				CLKPR = (1<<CLKPS2); // Clock division factor 16 (0100)
				
				PORTB &= ~(1<<ledPin);
			}
			else { // Not connected via USB, continue to normal functions
				
				setup_watchdog(T2S); // Sleep for 2 seconds
				system_sleep();
				if (buttonPressed != 1) { // Was the button released?
					int newbutton = buttonPressed;
					while (true) { // Sleep for 1 second after each button press, if no button press was made then exit
						setup_watchdog(T1S);
						system_sleep();
						if (newbutton != buttonPressed) {
							newbutton = buttonPressed;
						}
						else {
							break;
						}
					}
				}
				turnOffwatchdog();
				
				// User held down the button
				if (buttonPressed == 1) { // Change delay time
					cbi(GIMSK,PCIE); // Disable pin change interrupt (so that interrupt is not triggered when user releases button)
					functionSelect = CONFIGDELAY;
					delayTime = -1;
					blinkLed(1, T2S, SKIPLEDOFFDELAY);
					sbi(GIMSK,PCIE); // Re-enable pin change interrupt
				}
				
				// Start logging
				else if (buttonPressed == 6) { // Check if 6, as buttonPressed is incremented when user pushes and releases button
					functionSelect = STARTLOGGING;
					dataCount = 1;
					blinkLed(3, T500MS, 0);
				}
			}
		}
		
		// Log Temperature
		if (functionSelect == STARTLOGGING) {
			PORTB |= (1<<ledPin); // Turn on LED which also turns on the thermistor
			int tempValue = (int) thermistorTemp(analogRead(thermistorPin));
			PORTB &= ~(1<<ledPin); // OFF
			
			// Calculations to store -40C to 125C int to a single byte
			byte tempStore = tempValue + 130;
			if (tempValue <= -41) { // Temp is under min temp accepted
				tempStore = 90;
			}
			if (tempValue >= 126) { // Temp is over max temp accepted
				tempStore = 255;
			}
			
			// Write to EEPROM
			if (dataCount < 512) {
				eeprom_write_byte((uint8_t*) dataCount, tempStore);
				dataCount++;
				if (dataCount < 512) {
					eeprom_write_byte((uint8_t*) dataCount, 0); // So we know where to stop when extracting data
				}
			}
			else {
				functionSelect = WAITINPUT;
			}
			
			// Calculate the delay time
			int waitCounter = 0;
			int waitTime = 0;
			f_pin = 0;
			if (delayTime == 0) {
				waitTime = 15;
			}
			if (delayTime >= 1 && delayTime < 4) {
				waitTime = (delayTime * 5) * 15;
			}
			if (delayTime >= 4 && delayTime < 6) {
				waitTime = ((delayTime-3) * 30) * 15;
			}
			if (delayTime >= 6 && delayTime < 8) {
				waitTime = ((delayTime-5) * 120) * 15;
			}
			if (delayTime == 8) {
				waitTime = 8 * 60 * 15;
			}
			if (delayTime == 9) {
				waitTime = 12 * 60 * 15;
			}
			if (delayTime == 10) {
				waitTime = 24 * 60 * 15;
			}
			
			while (waitCounter != waitTime) {
				setup_watchdog(T4S);
				system_sleep();
				waitCounter++;
				if (f_pin == 1) { // Did a pin change wake us up?
					cbi(GIMSK,PCIE); // Disable pin change interrupt
					functionSelect = WAITINPUT;
					turnOffwatchdog(); // Disable the watchdog
					waitCounter = waitTime; // Break out of the loop
					blinkLed(2, T500MS, 0);
					sbi(GIMSK,PCIE); // Re-enable pin change interrupt
				}
			}
		}
		
		// Configure the delay time
		if (functionSelect == CONFIGDELAY) {
			system_sleep(); // Wait for next button press
			f_pin = 0;
			
			setup_watchdog(T1S);
			system_sleep();
			turnOffwatchdog();
			
			if (f_pin == 1) { // Button let go
				delayTime++;
				blinkLed(1, T500MS, SKIPLEDOFFDELAY);
				if (delayTime > 10) { // Incorrect value
					delayTime = -1;
					blinkLed(1, T2S, SKIPLEDOFFDELAY);
				}
			}
			
			// User held down the button to save setting
			else if (f_pin == 0) { 
				cbi(GIMSK,PCIE); // Disable pin change interrupt (so that interrupt is not triggered when user releases button)
				if (delayTime >= 0 && delayTime < 10) { // Save to EEPROM
					functionSelect = WAITINPUT;
					eeprom_write_byte((uint8_t*)0, delayTime);
					blinkLed(3, T500MS, 0);
				}
				else { // Incorrect value
					delayTime = -1;
					blinkLed(1, T2S, SKIPLEDOFFDELAY);
				}
				sbi(GIMSK,PCIE); // Re-enable pin change interrupt
			}
		}
	}
	return 0;
}

ISR(WDT_vect) { }

ISR(PCINT0_vect) {
	f_pin = 1; // Set global button pressed (or released) flag
	buttonPressed++;
}
